@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2006 Imendio AB
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +21,8 @@
  *          Xavier Claessens <xclaesse@gmail.com>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
 
@@ -29,10 +30,10 @@
 #include <glib/gi18n.h>
 #include <gobject/gvaluecollector.h>
 
-#include "libgossip-marshal.h"
+#include "gossip-account.h"
 #include "gossip-utils.h"
 
-#include "gossip-account.h"
+#include "libgossip-marshal.h"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_ACCOUNT, GossipAccountPriv))
 
@@ -45,6 +46,10 @@ struct _GossipAccountPriv {
 	gchar             *name;
 	gboolean           auto_connect;
 	gboolean           use_proxy;
+#ifdef HAVE_TELEPATHY
+	gchar             *protocol;
+	gchar             *cmgr_name;
+#endif
 
 	GHashTable        *parameters;
 };
@@ -76,26 +81,28 @@ static void           account_param_get_valist      (GossipAccount          *acc
 						     const gchar            *first_param_name,
 						     va_list                 var_args);
 static void           account_param_get_all_foreach (gchar                  *param_name,
+						     GossipAccountParam     *param,
+						     GList                 **params);
+static void           account_param_foreach_foreach (const gchar            *param_name,
 						     GossipAccountParam     *param,
 						     GossipAccountParamData *data);
 static void           account_param_free            (GossipAccountParam     *param);
 static void           account_set_type              (GossipAccount          *account,
 						     GossipAccountType       type);
+const gchar *         account_param_name_convert    (const gchar            *param_name);
 
 enum {
 	PROP_0,
 	PROP_TYPE,
 	PROP_NAME,
 	PROP_AUTO_CONNECT,
-	PROP_USE_PROXY
+	PROP_USE_PROXY,
+#ifdef HAVE_TELEPATHY
+	PROP_PROTOCOL,
+	PROP_CMGR_NAME,
+#endif
 };
 
-enum {
-	CHANGED,
-	LAST_SIGNAL
-};
-
-static guint     signals[LAST_SIGNAL] = {0};
 static gpointer  parent_class = NULL;
 
 GType
@@ -136,16 +143,6 @@ account_class_init (GossipAccountClass *class)
 	object_class->get_property = account_get_property;
 	object_class->set_property = account_set_property;
 
-	signals[CHANGED] =
-		g_signal_new ("changed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      libgossip_marshal_VOID__VOID,
-			      G_TYPE_NONE,
-			      0);
-
 	g_object_class_install_property (object_class,
 					 PROP_TYPE,
 					 g_param_spec_int ("type",
@@ -179,6 +176,23 @@ account_class_init (GossipAccountClass *class)
 							       "Identifies if the connection uses the environment proxy",
 							       FALSE,
 							       G_PARAM_READWRITE));
+#ifdef HAVE_TELEPATHY
+	g_object_class_install_property (object_class,
+					 PROP_PROTOCOL,
+					 g_param_spec_string ("protocol",
+							      "Protocol",
+							      "Protocol this account uses",
+							      NULL,
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class,
+					 PROP_CMGR_NAME,
+					 g_param_spec_string ("cmgr_name",
+							      "Connection Manager",
+							      "The connection manager that provides this account",
+							      NULL,
+							      G_PARAM_READWRITE));
+#endif
 
 	g_type_class_add_private (object_class, sizeof (GossipAccountPriv));
 }
@@ -187,16 +201,17 @@ static void
 account_init (GossipAccount *account)
 {
 	GossipAccountPriv *priv;
-	static gint        id = 1;
 
 	priv = GET_PRIV (account);
 
-	priv->unique_id    = id++;
-
 	priv->type         = 0;
 	priv->name         = NULL;
 	priv->auto_connect = TRUE;
 	priv->use_proxy    = FALSE;
+#ifdef HAVE_TELEPATHY
+	priv->protocol     = NULL;
+	priv->cmgr_name    = NULL;
+#endif
 
 	priv->parameters = g_hash_table_new_full (g_str_hash,
 						  g_str_equal,
@@ -212,6 +227,10 @@ account_finalize (GObject *object)
 	priv = GET_PRIV (object);
 
 	g_free (priv->name);
+#ifdef HAVE_TELEPATHY
+	g_free (priv->protocol);
+	g_free (priv->cmgr_name);
+#endif
 	g_hash_table_unref (priv->parameters);
 
 	(G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -242,6 +261,14 @@ account_get_property (GObject    *object,
 	case PROP_USE_PROXY:
 		g_value_set_boolean (value, priv->use_proxy);
 		break;
+#ifdef HAVE_TELEPATHY
+	case PROP_PROTOCOL:
+		g_value_set_string (value, priv->protocol);
+		break;
+	case PROP_CMGR_NAME:
+		g_value_set_string (value, priv->cmgr_name);
+		break;
+#endif
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -275,6 +302,16 @@ account_set_property (GObject      *object,
 		gossip_account_set_use_proxy (GOSSIP_ACCOUNT (object),
 					    g_value_get_boolean (value));
 		break;
+#ifdef HAVE_TELEPATHY
+	case PROP_PROTOCOL:
+		gossip_account_set_protocol (GOSSIP_ACCOUNT (object),
+					     g_value_get_string (value));
+		break;
+	case PROP_CMGR_NAME:
+		gossip_account_set_cmgr_name (GOSSIP_ACCOUNT (object),
+					      g_value_get_string (value));
+		break;
+#endif
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -296,12 +333,14 @@ account_param_new_valist (GossipAccount *account,
 		GType               g_type;
 		gchar              *error = NULL;
 
+		name = account_param_name_convert (name);
 		param = g_hash_table_lookup (priv->parameters, name);
 		if (param) {
 			g_warning ("GossipAccount already has a parameter named `%s'", name);
 			break;
 		}
-		param = g_new0 (GossipAccountParam, 1);
+
+		param = g_slice_new0 (GossipAccountParam);
 
 		g_type = va_arg (var_args, GType);
 		g_value_init (&param->g_value, g_type);
@@ -334,19 +373,28 @@ account_param_set_valist (GossipAccount *account,
 	for (name = first_param_name; name; name = va_arg (var_args, gchar*)) {
 		GossipAccountParam *param;
 		gchar              *error = NULL;
+		GValue              g_value = {0, };
 
+		name = account_param_name_convert (name);
 		param = g_hash_table_lookup (priv->parameters, name);
 		if (!param) {
 			g_warning ("GossipAccount has no parameter named `%s'", name);
 			break;
 		}
 
-		G_VALUE_COLLECT (&param->g_value, var_args, 0, &error);
+		g_value_init (&g_value, G_VALUE_TYPE (&param->g_value));
+		G_VALUE_COLLECT (&g_value, var_args, 0, &error);
 		if (error) {
+			g_value_unset (&g_value);
 			g_warning ("%s: %s", G_STRFUNC, error);
 			g_free (error);
 			break;
 		}
+		if (!gossip_g_value_equal (&g_value, &param->g_value)) {
+			param->modified = TRUE;
+			g_value_copy (&g_value, &param->g_value);
+		}
+		g_value_unset (&g_value);
 	}
 }
 
@@ -364,6 +412,7 @@ account_param_get_valist (GossipAccount *account,
 		GossipAccountParam *param;
 		gchar              *error = NULL;
 
+		name = account_param_name_convert (name);
 		param = g_hash_table_lookup (priv->parameters, name);
 		if (!param) {
 			g_warning ("GossipAccount has no parameter named `%s'", name);
@@ -391,8 +440,34 @@ gossip_account_param_new (GossipAccount *account,
 	va_start (var_args, first_param_name);
 	account_param_new_valist (account, first_param_name, var_args);
 	va_end (var_args);
+}
 
-	g_signal_emit (account, signals[CHANGED], 0);
+void
+gossip_account_param_new_g_value (GossipAccount           *account,
+				  const gchar             *param_name,
+				  const GValue            *g_value,
+				  GossipAccountParamFlags  flags)
+{
+	GossipAccountParam *param;
+	GossipAccountPriv  *priv;
+
+	priv = GET_PRIV (account);
+
+	param_name = account_param_name_convert (param_name);
+	param = g_hash_table_lookup (priv->parameters, param_name);
+	if (param) {
+		g_warning ("GossipAccount already has a parameter named `%s'", param_name);
+		return;
+	}
+	param = g_new0 (GossipAccountParam, 1);
+
+	g_value_init (&param->g_value, G_VALUE_TYPE (g_value));
+	g_value_copy (g_value, &param->g_value);
+	param->flags = flags;
+
+	g_hash_table_insert (priv->parameters,
+			     g_strdup (param_name),
+			     param);
 }
 
 void
@@ -407,14 +482,12 @@ gossip_account_param_set (GossipAccount *account,
 	va_start (var_args, first_param_name);
 	account_param_set_valist (account, first_param_name, var_args);
 	va_end (var_args);
-
-	g_signal_emit (account, signals[CHANGED], 0);
 }
 
 void
-gossip_account_param_set_g_value (GossipAccount           *account,
-				  const gchar             *param_name,
-				  const GValue            *g_value)
+gossip_account_param_set_g_value (GossipAccount *account,
+				  const gchar   *param_name,
+				  const GValue  *g_value)
 {
 	GossipAccountPriv  *priv;
 	GossipAccountParam *param;
@@ -423,15 +496,17 @@ gossip_account_param_set_g_value (GossipAccount           *account,
 
 	priv = GET_PRIV (account);
 
+	param_name = account_param_name_convert (param_name);
 	param = g_hash_table_lookup (priv->parameters, param_name);
 	if (!param) {
 		g_warning ("GossipAccount has no parameter named `%s'", param_name);
 		return;
 	}
 
-	g_value_copy (g_value, &param->g_value);
-
-	g_signal_emit (account, signals[CHANGED], 0);
+	if (!gossip_g_value_equal (g_value, &param->g_value)) {
+		param->modified = TRUE;
+		g_value_copy (g_value, &param->g_value);
+	}
 }
 
 void
@@ -459,6 +534,7 @@ gossip_account_param_get_g_value (GossipAccount *account,
 
 	priv = GET_PRIV (account);
 
+	param_name = account_param_name_convert (param_name);
 	param = g_hash_table_lookup (priv->parameters, param_name);
 	if (!param) {
 		g_warning ("GossipAccount has no parameter named `%s'", param_name);
@@ -478,9 +554,62 @@ gossip_account_has_param (GossipAccount *account,
 
 	priv = GET_PRIV (account);
 
+	param_name = account_param_name_convert (param_name);
 	return g_hash_table_lookup (priv->parameters, param_name) != NULL;
 }
 
+static void
+account_param_get_all_foreach (gchar               *param_name,
+			       GossipAccountParam  *param,
+			       GList              **params)
+{
+	*params = g_list_prepend (*params, g_strdup (account_param_name_convert (param_name)));
+}
+
+GList *
+gossip_account_param_get_all (GossipAccount *account)
+{
+	GossipAccountPriv *priv;
+	GList             *params = NULL;
+
+	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
+
+	priv = GET_PRIV (account);
+
+	g_hash_table_foreach (priv->parameters,
+			      (GHFunc) account_param_get_all_foreach,
+			      &params);
+
+	params = g_list_sort (params, (GCompareFunc) strcmp);
+
+	return params;
+}
+
+GossipAccountParam *
+gossip_account_param_get_param (GossipAccount *account,
+				const gchar   *param_name)
+{
+	GossipAccountPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
+
+	priv = GET_PRIV (account);
+
+	return g_hash_table_lookup (priv->parameters, param_name);
+}
+
+
+static void
+account_param_foreach_foreach (const gchar            *param_name,
+			       GossipAccountParam     *param,
+			       GossipAccountParamData *data)
+{
+	data->callback (data->account,
+			account_param_name_convert (param_name),
+			param,
+			data->user_data);
+}
+
 void
 gossip_account_param_foreach (GossipAccount           *account,
 			      GossipAccountParamFunc   callback,
@@ -499,26 +628,15 @@ gossip_account_param_foreach (GossipAccount           *account,
 	data->account = account;
 
 	g_hash_table_foreach (priv->parameters,
-			      (GHFunc) account_param_get_all_foreach,
+			      (GHFunc) account_param_foreach_foreach,
 			      data);
 }
 
-static void
-account_param_get_all_foreach (gchar                  *param_name,
-			       GossipAccountParam     *param,
-			       GossipAccountParamData *data)
-{
-	data->callback (data->account,
-			param_name,
-			param,
-			data->user_data);
-}
-
 static void
 account_param_free (GossipAccountParam *param)
 {
 	g_value_unset (&param->g_value);
-	g_free (param);
+	g_slice_free (GossipAccountParam, param);
 }
 
 GossipAccountType
@@ -575,8 +693,6 @@ account_set_type (GossipAccount     *account,
 
 	priv = GET_PRIV (account);
 	priv->type = type;
-
-	g_signal_emit (account, signals[CHANGED], 0);
 }
 
 void
@@ -594,7 +710,6 @@ gossip_account_set_name (GossipAccount *account,
 	priv->name = g_strdup (name);
 
 	g_object_notify (G_OBJECT (account), "name");
-	g_signal_emit (account, signals[CHANGED], 0);
 }
 
 void
@@ -609,7 +724,6 @@ gossip_account_set_auto_connect (GossipAccount *account,
 	priv->auto_connect = auto_connect;
 
 	g_object_notify (G_OBJECT (account), "auto_connect");
-	g_signal_emit (account, signals[CHANGED], 0);
 }
 
 void
@@ -624,35 +738,77 @@ gossip_account_set_use_proxy (GossipAccount *account,
 	priv->use_proxy = use_proxy;
 
 	g_object_notify (G_OBJECT (account), "use_proxy");
-	g_signal_emit (account, signals[CHANGED], 0);
 }
 
+#ifdef HAVE_TELEPATHY
+const gchar *
+gossip_account_get_protocol (GossipAccount *account)
+{
+	GossipAccountPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
+
+	priv = GET_PRIV (account);
+	return priv->protocol;
+}
+
+const gchar *
+gossip_account_get_cmgr_name (GossipAccount *account)
+{
+	GossipAccountPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
+
+	priv = GET_PRIV (account);
+	return priv->cmgr_name;
+}
+
+void 
+gossip_account_set_protocol (GossipAccount *account,
+			     const gchar   *protocol)
+{
+	GossipAccountPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
+	g_return_if_fail (protocol != NULL);
+	
+	priv = GET_PRIV (account);
+	
+	g_free (priv->protocol);
+	priv->protocol = g_strdup (protocol);
+
+	g_object_notify (G_OBJECT (account), "protocol");
+}
+
+void 
+gossip_account_set_cmgr_name (GossipAccount *account,
+			 const gchar   *cmgr_name)
+{
+	GossipAccountPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
+	g_return_if_fail (cmgr_name != NULL);
+	
+	priv = GET_PRIV (account);
+	
+	g_free (priv->cmgr_name);
+	priv->cmgr_name = g_strdup (cmgr_name);
+
+	g_object_notify (G_OBJECT (account), "cmgr_name");
+}
+#endif
+
 guint
 gossip_account_hash (gconstpointer key)
 {
-	GossipAccountPriv *priv;
-
-	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (key), 0);
-
-	priv = GET_PRIV (key);
-
-	return g_int_hash (&priv->unique_id);
+	return g_direct_hash (key);
 }
 
 gboolean
 gossip_account_equal (gconstpointer a,
 		      gconstpointer b)
 {
-	GossipAccountPriv *priv1;
-	GossipAccountPriv *priv2;
-
-	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (a), FALSE);
-	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (b), FALSE);
-
-	priv1 = GET_PRIV (a);
-	priv2 = GET_PRIV (b);
-
-	return (priv1->unique_id == priv2->unique_id);
+	return g_direct_equal (a, b);
 }
 
 const gchar *
@@ -694,3 +850,15 @@ gossip_account_string_to_type (const gchar *str)
 
 	return GOSSIP_ACCOUNT_TYPE_UNKNOWN;
 }
+
+const gchar *
+account_param_name_convert (const gchar *param_name)
+{
+#ifdef HAVE_TELEPATHY
+	if (strcmp (param_name, "use_ssl") == 0) {
+		return "old-ssl";
+	}
+#endif
+	return param_name;
+}
+
