@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004 Imendio AB
+ * Copyright (C) 2004-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,9 +16,12 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
 
 #include "gossip-vcard.h"
 
@@ -27,14 +30,14 @@
 typedef struct _GossipVCardPriv GossipVCardPriv;
 
 struct _GossipVCardPriv {
-	gchar    *name;
-	gchar    *nickname;
-	gchar    *email;
-	gchar    *url;
-	gchar    *country;
-	gchar    *description;
-	gpointer *avatar;
-	guint     avatar_size;
+	gchar        *name;
+	gchar        *nickname;
+	gchar        *birthday;
+	gchar        *email;
+	gchar        *url;
+	gchar        *country;
+	gchar        *description;
+	GossipAvatar *avatar;
 };
 
 static void vcard_finalize     (GObject      *object);
@@ -51,12 +54,12 @@ enum {
 	PROP_0,
 	PROP_NAME,
 	PROP_NICKNAME,
+	PROP_BIRTHDAY,
 	PROP_EMAIL,
 	PROP_URL,
 	PROP_COUNTRY,
 	PROP_DESCRIPTION,
 	PROP_AVATAR,
-	PROP_AVATAR_SIZE
 };
 
 G_DEFINE_TYPE (GossipVCard, gossip_vcard, G_TYPE_OBJECT);
@@ -86,6 +89,13 @@ gossip_vcard_class_init (GossipVCardClass *class)
 							      "The nickname field",
 							      NULL,
 							      G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_BIRTHDAY,
+					 g_param_spec_string ("birthday",
+							      "Birthday field",
+							      "The birthday field",
+							      NULL,
+							      G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
 					 PROP_EMAIL,
 					 g_param_spec_string ("email",
@@ -121,16 +131,6 @@ gossip_vcard_class_init (GossipVCardClass *class)
 							       "The avatar image",
 							       G_PARAM_READWRITE));
 
-	g_object_class_install_property (object_class,
-					 PROP_AVATAR_SIZE,
-					 g_param_spec_uint ("avatar_size",
-							    "Avatar size",
-							    "The size of the avatar image",
-							    0,
-							    G_MAXUINT,
-							    0,
-							    G_PARAM_READWRITE));
-
 	g_type_class_add_private (object_class, sizeof (GossipVCardPriv));
 }
 
@@ -143,12 +143,12 @@ gossip_vcard_init (GossipVCard *vcard)
 
 	priv->name        = NULL;
 	priv->nickname    = NULL;
+	priv->birthday    = NULL;
 	priv->email       = NULL;
 	priv->url         = NULL;
 	priv->country     = NULL;
 	priv->description = NULL;
 	priv->avatar      = NULL;
-	priv->avatar_size = 0;
 }
 
 static void
@@ -160,11 +160,15 @@ vcard_finalize (GObject *object)
 
 	g_free (priv->name);
 	g_free (priv->nickname);
+	g_free (priv->birthday);
 	g_free (priv->email);
 	g_free (priv->url);
 	g_free (priv->country);
 	g_free (priv->description);
-	g_free (priv->avatar);
+
+	if (priv->avatar) {
+		gossip_avatar_unref (priv->avatar);
+	}
 
 	(G_OBJECT_CLASS (gossip_vcard_parent_class)->finalize) (object);
 }
@@ -186,6 +190,9 @@ vcard_get_property (GObject    *object,
 	case PROP_NICKNAME:
 		g_value_set_string (value, priv->nickname);
 		break;
+	case PROP_BIRTHDAY:
+		g_value_set_string (value, priv->birthday);
+		break;
 	case PROP_EMAIL:
 		g_value_set_string (value, priv->email);
 		break;
@@ -201,9 +208,6 @@ vcard_get_property (GObject    *object,
 	case PROP_AVATAR:
 		g_value_set_pointer (value, priv->avatar);
 		break;
-	case PROP_AVATAR_SIZE:
-		g_value_set_uint (value, priv->avatar_size);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -229,6 +233,10 @@ vcard_set_property (GObject      *object,
 		gossip_vcard_set_nickname (GOSSIP_VCARD (object),
 					   g_value_get_string (value));
 		break;
+	case PROP_BIRTHDAY:
+		gossip_vcard_set_birthday (GOSSIP_VCARD (object),
+					   g_value_get_string (value));
+		break;
 	case PROP_EMAIL:
 		gossip_vcard_set_email (GOSSIP_VCARD (object),
 					g_value_get_string (value));
@@ -246,10 +254,8 @@ vcard_set_property (GObject      *object,
 					      g_value_get_string (value));
 		break;
 	case PROP_AVATAR:
-		priv->avatar = g_value_get_pointer (value);
-		break;
-	case PROP_AVATAR_SIZE:
-		priv->avatar_size = g_value_get_uint (value);
+		gossip_vcard_set_avatar (GOSSIP_VCARD (object),
+					 g_value_get_pointer (value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -287,6 +293,18 @@ gossip_vcard_get_nickname (GossipVCard *vcard)
 	return priv->nickname;
 }
 
+const gchar *
+gossip_vcard_get_birthday (GossipVCard *vcard)
+{
+	GossipVCardPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
+
+	priv = GOSSIP_VCARD_GET_PRIV (vcard);
+
+	return priv->birthday;
+}
+
 const gchar *
 gossip_vcard_get_email (GossipVCard *vcard)
 {
@@ -335,22 +353,16 @@ gossip_vcard_get_description (GossipVCard *vcard)
 	return priv->description;
 }
 
-const guchar *
-gossip_vcard_get_avatar (GossipVCard *vcard,
-			 gsize       *avatar_size)
+GossipAvatar *
+gossip_vcard_get_avatar (GossipVCard *vcard)
 {
 	GossipVCardPriv *priv;
-	guchar          *avatar;
 
 	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
-	g_return_val_if_fail (avatar_size != NULL, NULL);
 
 	priv = GOSSIP_VCARD_GET_PRIV (vcard);
 
-	avatar = (guchar*) priv->avatar;
-	*avatar_size = priv->avatar_size;
-
-	return avatar;
+	return priv->avatar;
 }
 
 
@@ -392,6 +404,24 @@ gossip_vcard_set_nickname (GossipVCard *vcard,
 	}
 }
 
+void
+gossip_vcard_set_birthday (GossipVCard *vcard,
+			   const gchar *birthday)
+{
+	GossipVCardPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_VCARD (vcard));
+
+	priv = GOSSIP_VCARD_GET_PRIV (vcard);
+
+	g_free (priv->birthday);
+	if (birthday) {
+		priv->birthday = g_strdup (birthday);
+	} else {
+		priv->birthday = NULL;
+	}
+}
+
 void
 gossip_vcard_set_email (GossipVCard *vcard,
 			const gchar *email)
@@ -470,8 +500,7 @@ gossip_vcard_set_description (GossipVCard *vcard,
 
 void
 gossip_vcard_set_avatar (GossipVCard  *vcard,
-			 const guchar *avatar,
-			 gsize         avatar_size)
+			 GossipAvatar *avatar)
 {
 	GossipVCardPriv *priv;
 
@@ -479,13 +508,13 @@ gossip_vcard_set_avatar (GossipVCard  *vcard,
 
 	priv = GOSSIP_VCARD_GET_PRIV (vcard);
 
-	g_free (priv->avatar);
+	if (priv->avatar) {
+		gossip_avatar_unref (priv->avatar);
+		priv->avatar = NULL;
+	}
 
 	if (avatar) {
-		priv->avatar = g_memdup (avatar, avatar_size);
-		priv->avatar_size = avatar_size;
-	} else {
-		priv->avatar = NULL;
-		priv->avatar_size = 0;
+		priv->avatar = gossip_avatar_ref (avatar);
 	}
 }
+
