@@ -18,18 +18,24 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <stdlib.h>
 #include <string.h>
 
-#include <libgossip/gossip-utils.h>
+
 #include <libgossip/gossip-debug.h>
-#include <libgossip/gossip-chatroom-contact.h>
+#include <libgossip/gossip-chatroom.h>
+#include <libgossip/gossip-ft.h>
+#include <libgossip/gossip-ft-provider.h>
+#include <libgossip/gossip-message.h>
+#include <libgossip/gossip-utils.h>
 
 #include "gossip-jabber-chatrooms.h"
-#include "gossip-jid.h"
+#include "gossip-jabber-disco.h"
 #include "gossip-jabber-utils.h"
 #include "gossip-jabber-private.h"
+#include "gossip-jid.h"
 
 #define DEBUG_DOMAIN "JabberChatrooms"
 
@@ -66,39 +72,48 @@ typedef struct {
 	LmMessageHandler      *join_handler;
 } JabberChatroom;
 
-static void             jabber_chatrooms_logged_out_cb         (GossipProtocol         *jabber,
-								GossipAccount          *account,
-								gint                    reason,
-								GossipJabberChatrooms  *chatrooms);
-static JabberChatroom * jabber_chatrooms_chatroom_new          (GossipJabberChatrooms  *chatrooms,
-								GossipChatroom         *chatroom);
-static JabberChatroom * jabber_chatrooms_chatroom_ref          (JabberChatroom         *room);
-static void             jabber_chatrooms_chatroom_unref        (JabberChatroom         *room);
-static GossipChatroomId jabber_chatrooms_chatroom_get_id       (JabberChatroom         *room);
-static GossipContact *  jabber_chatrooms_get_contact           (JabberChatroom         *room,
-								GossipJID              *jid,
-								gboolean               *new_contact);
-static LmHandlerResult  jabber_chatrooms_message_handler       (LmMessageHandler       *handler,
-								LmConnection           *conn,
-								LmMessage              *message,
-								GossipJabberChatrooms  *chatrooms);
-static LmHandlerResult  jabber_chatrooms_presence_handler      (LmMessageHandler       *handler,
-								LmConnection           *conn,
-								LmMessage              *message,
-								GossipJabberChatrooms  *chatrooms);
-static LmHandlerResult  jabber_chatrooms_join_cb               (LmMessageHandler       *handler,
-								LmConnection           *connection,
-								LmMessage              *message,
-								JabberChatroom         *room);
-static void             jabber_chatrooms_get_rooms_foreach     (gpointer                key,
-								JabberChatroom         *room,
-								GList                 **list);
-static void             jabber_chatrooms_set_presence_foreach  (gpointer                key,
-								JabberChatroom         *room,
-								GossipJabberChatrooms  *chatrooms);
-static LmMessageNode *  jabber_chatrooms_find_muc_user_node    (LmMessageNode          *parent_node);
-static GossipChatroomRole jabber_chatrooms_get_role            (LmMessageNode          *muc_node);
-static GossipChatroomAffiliation jabber_chatrooms_get_affiliation (LmMessageNode       *muc_node);
+static void             jabber_chatrooms_logged_out_cb        (GossipProtocol         *jabber,
+							       GossipAccount          *account,
+							       gint                    reason,
+							       GossipJabberChatrooms  *chatrooms);
+static JabberChatroom * jabber_chatrooms_chatroom_new         (GossipJabberChatrooms  *chatrooms,
+							       GossipChatroom         *chatroom);
+static JabberChatroom * jabber_chatrooms_chatroom_ref         (JabberChatroom         *room);
+static void             jabber_chatrooms_chatroom_unref       (JabberChatroom         *room);
+static GossipChatroomId jabber_chatrooms_chatroom_get_id      (JabberChatroom         *room);
+static GossipContact *  jabber_chatrooms_get_contact          (JabberChatroom         *room,
+							       GossipJID              *jid,
+							       gboolean               *new_contact);
+static LmHandlerResult  jabber_chatrooms_message_handler      (LmMessageHandler       *handler,
+							       LmConnection           *conn,
+							       LmMessage              *message,
+							       GossipJabberChatrooms  *chatrooms);
+static LmHandlerResult  jabber_chatrooms_presence_handler     (LmMessageHandler       *handler,
+							       LmConnection           *conn,
+							       LmMessage              *message,
+							       GossipJabberChatrooms  *chatrooms);
+static LmHandlerResult  jabber_chatrooms_join_cb              (LmMessageHandler       *handler,
+							       LmConnection           *connection,
+							       LmMessage              *message,
+							       JabberChatroom         *room);
+static void             jabber_chatrooms_get_rooms_foreach    (gpointer                key,
+							       JabberChatroom         *room,
+							       GList                 **list);
+static void             jabber_chatrooms_browse_rooms_cb      (GossipJabberDisco      *disco,
+							       GossipJabberDiscoItem  *item,
+							       gboolean                last_item,
+							       gboolean                timeout,
+							       GError                 *error,
+							       GossipCallbackData     *data);
+static void             jabber_chatrooms_set_presence_foreach (gpointer                key,
+							       JabberChatroom         *room,
+							       GossipJabberChatrooms  *chatrooms);
+
+static LmMessageNode *  jabber_chatrooms_find_muc_user_node   (LmMessageNode          *parent_node);
+static GossipChatroomRole 
+                        jabber_chatrooms_get_role             (LmMessageNode          *muc_node);
+static GossipChatroomAffiliation 
+                        jabber_chatrooms_get_affiliation      (LmMessageNode       *muc_node);
 
 GossipJabberChatrooms *
 gossip_jabber_chatrooms_init (GossipJabber *jabber)
@@ -124,7 +139,7 @@ gossip_jabber_chatrooms_init (GossipJabber *jabber)
 							  (GDestroyNotify) gossip_jid_unref,
 							  NULL);
 
-	g_signal_connect (GOSSIP_PROTOCOL (chatrooms->jabber), "logged-out",
+	g_signal_connect (GOSSIP_PROTOCOL (chatrooms->jabber), "disconnected",
 			  G_CALLBACK (jabber_chatrooms_logged_out_cb),
 			  chatrooms);
 
@@ -311,7 +326,6 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 	LmMessageNode    *node;
 	const gchar      *from;
 
-
 	if (lm_message_get_sub_type (m) != LM_MESSAGE_SUB_TYPE_GROUPCHAT) {
 		return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 	}
@@ -334,7 +348,7 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 					       "chatroom-new-event",
 					       id, node->value);
 		} else {
-			gossip_time_t timestamp;
+			GossipTime timestamp;
 
 			timestamp = gossip_jabber_get_message_timestamp (m);
 
@@ -404,9 +418,12 @@ jabber_chatrooms_get_contact (JabberChatroom *room,
 		name = id;
 	}
 
-	contact = GOSSIP_CONTACT (gossip_chatroom_contact_new_full (gossip_contact_get_account (room->own_contact),
-								    id,
-								    name));
+	contact = g_object_new (GOSSIP_TYPE_CONTACT,
+				"account", gossip_contact_get_account (room->own_contact),
+				"id", id,
+				"name", name,
+				NULL);
+
 	room->contacts = g_slist_prepend (room->contacts, contact);
 
 	return contact;
@@ -430,8 +447,7 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 	gboolean                   new_contact;
 	gboolean                   was_offline;
 	LmMessageNode             *muc_user_node;
-	GossipChatroomRole         muc_role;
-	GossipChatroomAffiliation  muc_affiliation;
+	GossipChatroomContactInfo  muc_contact_info;
 
 	from = lm_message_node_get_attribute (m->node, "from");
 	jid = gossip_jid_new (from);
@@ -467,31 +483,23 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 		/* should signal joined if contact was found but offline */
 		was_offline = !gossip_contact_is_online (contact);
 		gossip_contact_add_presence (contact, presence);
+		g_object_unref (presence);
 
 		muc_user_node = jabber_chatrooms_find_muc_user_node (m->node);
-		muc_role = jabber_chatrooms_get_role (muc_user_node);
-		muc_affiliation = jabber_chatrooms_get_affiliation (muc_user_node);
-
-		g_object_set (contact,
-			      "role", muc_role,
-			      "affiliation", muc_affiliation,
-			      NULL);
-
-		g_object_unref (presence);
+		muc_contact_info.role = jabber_chatrooms_get_role (muc_user_node);
+		muc_contact_info.affiliation = jabber_chatrooms_get_affiliation (muc_user_node);
 
 		/* is contact new or updated */
 		if (new_contact || was_offline) {
 			gossip_debug (DEBUG_DOMAIN, "ID[%d] Presence for new joining contact:'%s'",
 				      id, gossip_jid_get_full (jid));
-			g_signal_emit_by_name (chatrooms->jabber,
-					       "chatroom-contact-joined",
-					       id, contact);
+			gossip_chatroom_contact_joined (room->chatroom,
+							contact,
+							&muc_contact_info);
 		} else {
-			gossip_debug (DEBUG_DOMAIN, "ID[%d] Presence updated for contact:'%s'",
-				      id, gossip_jid_get_full (jid));
-			g_signal_emit_by_name (chatrooms->jabber,
-					       "chatroom-contact-presence-updated",
-					       id, contact);
+			gossip_chatroom_set_contact_info (room->chatroom,
+							  contact,
+							  &muc_contact_info);
 		}
 		break;
 
@@ -500,9 +508,7 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 		if (contact) {
 			gossip_debug (DEBUG_DOMAIN, "ID[%d] Contact left:'%s'",
 				      id, gossip_jid_get_full (jid));
-			g_signal_emit_by_name (chatrooms->jabber,
-					       "chatroom-contact-left",
-					       id, contact);
+			gossip_chatroom_contact_left (room->chatroom, contact);
 			room->contacts = g_slist_remove (room->contacts, contact);
 			g_object_unref (contact);
 		}
@@ -662,7 +668,8 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
 						LM_HANDLER_PRIORITY_FIRST);
 
 	/* Some servers don't honor the id so we don't get a reply and
-	   are waiting forever. */
+	 * are waiting forever. 
+	 */
 	lm_connection_send (chatrooms->connection, m,  NULL);
 
 	lm_message_unref (m);
@@ -835,7 +842,6 @@ gossip_jabber_chatrooms_cancel (GossipJabberChatrooms *chatrooms,
 		room->join_handler = NULL;
 	}
 
-
 	gossip_chatroom_set_status (room->chatroom, GOSSIP_CHATROOM_STATUS_INACTIVE);
 	gossip_chatroom_set_last_error (room->chatroom, NULL);
 
@@ -985,8 +991,8 @@ gossip_jabber_chatrooms_leave (GossipJabberChatrooms *chatrooms,
 }
 
 GossipChatroom *
-gossip_jabber_chatrooms_find (GossipJabberChatrooms *chatrooms,
-			      GossipChatroomId       id)
+gossip_jabber_chatrooms_find_by_id (GossipJabberChatrooms *chatrooms,
+				    GossipChatroomId       id)
 {
 	JabberChatroom *room;
 
@@ -1001,6 +1007,33 @@ gossip_jabber_chatrooms_find (GossipJabberChatrooms *chatrooms,
 	return room->chatroom;
 }
 
+GossipChatroom *
+gossip_jabber_chatrooms_find (GossipJabberChatrooms *chatrooms,
+			      GossipChatroom        *chatroom)
+{
+	JabberChatroom *room;
+	GossipJID      *jid;
+	gchar          *jid_str;
+
+	g_return_val_if_fail (chatrooms != NULL, NULL);
+	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), NULL);
+
+	jid_str = g_strdup_printf ("%s/%s",
+				   gossip_chatroom_get_id_str (chatroom),
+				   gossip_chatroom_get_nick (chatroom));
+	jid = gossip_jid_new (jid_str);
+	g_free (jid_str);
+
+	room = g_hash_table_lookup (chatrooms->room_jid_hash, jid);
+	gossip_jid_unref (jid);
+
+	if (!room) {
+		return NULL;
+	}
+
+	return room->chatroom;
+}
+
 void
 gossip_jabber_chatrooms_invite (GossipJabberChatrooms *chatrooms,
 				GossipChatroomId       id,
@@ -1134,6 +1167,14 @@ gossip_jabber_chatrooms_invite_decline (GossipJabberChatrooms *chatrooms,
 	lm_message_unref (m);
 }
 
+static void
+jabber_chatrooms_get_rooms_foreach (gpointer               key,
+				    JabberChatroom        *room,
+				    GList                **list)
+{
+	*list = g_list_append (*list, key);
+}
+
 GList *
 gossip_jabber_chatrooms_get_rooms (GossipJabberChatrooms *chatrooms)
 {
@@ -1149,11 +1190,110 @@ gossip_jabber_chatrooms_get_rooms (GossipJabberChatrooms *chatrooms)
 }
 
 static void
-jabber_chatrooms_get_rooms_foreach (gpointer               key,
-				    JabberChatroom        *room,
-				    GList                **list)
+jabber_chatrooms_browse_rooms_cb (GossipJabberDisco     *disco,
+				  GossipJabberDiscoItem *item,
+				  gboolean               last_item,
+				  gboolean               timeout,
+				  GError                *error,
+				  GossipCallbackData    *data)
 {
-	*list = g_list_append (*list, key);
+	GossipJID             *jid = NULL;
+	GossipJabberChatrooms *chatrooms;
+	GossipChatroom        *chatroom = NULL;
+	JabberChatroom        *room = NULL;
+	GList                 *list;
+	gchar                 *server;
+
+	if (timeout && !last_item) {
+		return;
+	}
+
+	chatrooms = data->data1;
+	server = data->data2;
+	list = data->data3;
+
+	if (item) {
+		jid = gossip_jabber_disco_item_get_jid (item);
+		room = g_hash_table_lookup (chatrooms->room_jid_hash, jid);
+	}
+
+	if (room) {
+		chatroom = room->chatroom;
+	} 
+
+	if (!room && !timeout && !error) {
+		GossipAccount  *account;
+		const gchar    *server;
+		const gchar    *name;
+		gchar          *room;
+		
+		gossip_debug (DEBUG_DOMAIN, 
+			      "Chatroom found on server not set up here, creating for:'%s'...",
+			      gossip_jid_get_full (jid));
+
+		account = gossip_jabber_get_account (chatrooms->jabber);
+		server = gossip_jid_get_part_host (jid);
+		room = gossip_jid_get_part_name (jid);
+		name = gossip_jabber_disco_item_get_name (item);
+
+		/* Create new chatroom */
+		chatroom = g_object_new (GOSSIP_TYPE_CHATROOM,
+					 "type", GOSSIP_CHATROOM_TYPE_NORMAL,
+					 "account", account,
+					 "server", server,
+					 "name", room,
+					 "room", room,
+					 NULL);
+		g_free (room);
+	}
+
+	if (chatroom) {
+		gossip_debug (DEBUG_DOMAIN, 
+			      "Chatroom:'%s' added to list found on server:'%s'...",
+			      gossip_chatroom_get_room (chatroom),
+			      gossip_chatroom_get_server (chatroom));
+
+		list = g_list_prepend (list, chatroom);
+		data->data3 = list;
+	}
+
+	if (last_item) {
+		GossipChatroomBrowseCb callback;
+		
+		callback = data->callback;
+		(callback)(GOSSIP_CHATROOM_PROVIDER (chatrooms->jabber),
+			   server, list, error, data->user_data);
+
+		g_list_foreach (list, (GFunc) g_object_unref, NULL);
+		g_list_free (list);
+
+		g_free (server);
+
+		g_free (data);
+	}
+}
+
+void
+gossip_jabber_chatrooms_browse_rooms (GossipJabberChatrooms  *chatrooms,
+				      const gchar            *server,
+				      GossipChatroomBrowseCb  callback,
+				      gpointer                user_data)
+{
+	GossipJabberDisco  *disco;
+	GossipCallbackData *data;
+	
+	data = g_new0 (GossipCallbackData, 1);
+	
+	data->callback = callback;
+	data->user_data = user_data;
+	data->data1 = chatrooms;
+	data->data2 = g_strdup (server);
+
+	disco = gossip_jabber_disco_request (chatrooms->jabber,
+					     server,
+					     (GossipJabberDiscoItemFunc) 
+					     jabber_chatrooms_browse_rooms_cb,
+					     data);
 }
 
 static void
