@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2006 Imendio AB
- * Copyright (C) 2004-2005 Martyn Russell <mr@gnome.org>
- * Copyright (C) 2003      Kevin Dougherty <gossip@kdough.net>
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,25 +16,34 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
+ *          Kevin Dougherty <gossip@kdough.net>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdlib.h>
 #include <string.h>
+
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
+#include <libgossip/gossip-chatroom.h>
 #include <libgossip/gossip-contact.h>
-#include <libgossip/gossip-debug.h>
-#include <libgossip/gossip-ft.h>
+#include <libgossip/gossip-event.h>
 #include <libgossip/gossip-presence.h>
+#include <libgossip/gossip-chatroom-provider.h>
 #include <libgossip/gossip-protocol.h>
-#include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-account-manager.h>
 #include <libgossip/gossip-conf.h>
+#include <libgossip/gossip-debug.h>
 
 #include "gossip-about-dialog.h"
 #include "gossip-accounts-dialog.h"
@@ -60,9 +67,9 @@
 #include "gossip-status-presets.h"
 #include "gossip-stock.h"
 #include "gossip-subscription-dialog.h"
-#include "gossip-throbber.h"
 #include "gossip-ui-utils.h"
 #include "gossip-vcard-dialog.h"
+#include "ephy-spinner.h"
 
 #ifdef HAVE_DBUS
 #include "gossip-dbus.h"
@@ -177,8 +184,7 @@ struct _GossipAppPriv {
 static void            gossip_app_class_init                  (GossipAppClass           *klass);
 static void            gossip_app_init                        (GossipApp                *singleton_app);
 static void            app_finalize                           (GObject                  *object);
-static void            app_setup                              (GossipSession            *session,
-							       GossipAccountManager     *manager);
+static void            app_setup                              (GossipSession            *session);
 static gboolean        app_main_window_quit_confirm           (GossipApp                *app,
 							       GtkWidget                *window);
 static void            app_main_window_quit_confirm_cb        (GtkWidget                *dialog,
@@ -322,7 +328,7 @@ static gboolean        app_window_configure_event_cb          (GtkWidget
 							       GossipApp                *app);
 static void            app_status_flash_start                 (void);
 static void            app_status_flash_stop                  (void);
-static void            app_chatroom_auto_connect_cb           (GossipChatroomManager    *manager,
+static void            app_session_chatroom_auto_connect_cb   (GossipSession            *session,
 							       GossipChatroomProvider   *provider,
 							       GossipChatroom           *chatroom,
 							       gpointer                  user_data);
@@ -377,9 +383,8 @@ gossip_app_init (GossipApp *singleton_app)
 static void
 app_finalize (GObject *object)
 {
-	GossipApp            *app;
-	GossipAppPriv        *priv;
-	GossipAccountManager *manager;
+	GossipApp     *app;
+	GossipAppPriv *priv;
 
 	app = GOSSIP_APP (object);
 	priv = GET_PRIV (app);
@@ -421,8 +426,6 @@ app_finalize (GObject *object)
 
 	gossip_sound_finalize ();
 
-	manager = gossip_session_get_account_manager (priv->session);
-
 	g_signal_handlers_disconnect_by_func (priv->session,
 					      app_session_protocol_connecting_cb,
 					      NULL);
@@ -438,9 +441,8 @@ app_finalize (GObject *object)
 	g_signal_handlers_disconnect_by_func (priv->session,
 					      app_session_get_password_cb,
 					      NULL);
-
-	g_signal_handlers_disconnect_by_func (priv->chatroom_manager,
-					      app_chatroom_auto_connect_cb,
+	g_signal_handlers_disconnect_by_func (priv->session,
+					      app_session_chatroom_auto_connect_cb,
 					      NULL);
 
 	g_signal_handlers_disconnect_by_func (priv->event_manager,
@@ -458,32 +460,32 @@ app_finalize (GObject *object)
 		g_object_unref (priv->away_presence);
 	}
 
+	gossip_ft_window_finalize (priv->session);
+	gossip_subscription_dialog_finalize (priv->session);
+
 	g_object_unref (priv->event_manager);
 	g_object_unref (priv->chat_manager);
 	g_object_unref (priv->chatroom_manager);
 	g_object_unref (priv->session);
 
-	gossip_ft_window_finalize (priv->session);
-	gossip_subscription_dialog_finalize (priv->session);
-
 	G_OBJECT_CLASS (gossip_app_parent_class)->finalize (object);
 }
 
 static void
-app_setup (GossipSession        *session,
-	   GossipAccountManager *manager)
+app_setup (GossipSession *session)
 {
-	GossipAppPriv  *priv;
-	GossipConf     *conf;
-	GladeXML       *glade;
-	GtkWidget      *sw;
-	GtkWidget      *show_offline_widget;
-	GtkToolItem    *item;
-	gchar          *str;
-	gboolean        show_offline;
-	gboolean        show_avatars;
-	gboolean        compact_contact_list;
-	gint            x, y, w, h;
+	GossipAppPriv *priv;
+	GossipConf    *conf;
+	GladeXML      *glade;
+	GtkWidget     *sw;
+	GtkWidget     *show_offline_widget;
+	GtkWidget     *ebox;
+	GtkToolItem   *item;
+	gchar         *str;
+	gboolean       show_offline;
+	gboolean       show_avatars;
+	gboolean       compact_contact_list;
+	gint           x, y, w, h;
 
 	gossip_debug (DEBUG_DOMAIN_SETUP, "Beginning...");
 
@@ -510,9 +512,7 @@ app_setup (GossipSession        *session,
 		      "Initialising managers "
 		      "(chatroom, chat, event)");
 
-	priv->chatroom_manager = gossip_chatroom_manager_new (manager,
-							      priv->session,
-							      NULL);
+	priv->chatroom_manager = gossip_session_get_chatroom_manager (priv->session);
 
 	priv->chat_manager = gossip_chat_manager_new ();
 	priv->event_manager = gossip_event_manager_new ();
@@ -548,8 +548,8 @@ app_setup (GossipSession        *session,
 			  G_CALLBACK (app_session_get_password_cb),
 			  NULL);
 
-	g_signal_connect (priv->chatroom_manager, "chatroom-auto-connect",
-			  G_CALLBACK (app_chatroom_auto_connect_cb),
+	g_signal_connect (priv->session, "chatroom-auto-connect",
+			  G_CALLBACK (app_session_chatroom_auto_connect_cb),
 			  NULL);
 
 	g_signal_connect (priv->event_manager, "event-added",
@@ -636,20 +636,24 @@ app_setup (GossipSession        *session,
 						  priv->presence_chooser);
 
 	/* Set up the throbber */
-	priv->throbber = gossip_throbber_new ();
-	gtk_widget_show (priv->throbber);
+	ebox = gtk_event_box_new ();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX (ebox), FALSE);
+
+	priv->throbber = ephy_spinner_new ();
+	ephy_spinner_set_size (EPHY_SPINNER (priv->throbber), GTK_ICON_SIZE_LARGE_TOOLBAR);
+	gtk_container_add (GTK_CONTAINER (ebox), priv->throbber);
 
 	item = gtk_tool_item_new ();
-	gtk_container_add (GTK_CONTAINER (item), priv->throbber);
-	gtk_widget_show (GTK_WIDGET (item));
+	gtk_container_add (GTK_CONTAINER (item), ebox);
+	gtk_widget_show_all (GTK_WIDGET (item));
 
 	gtk_toolbar_insert (GTK_TOOLBAR (priv->presence_toolbar), item, -1);
 
 	str = _("Show and edit accounts");
 	gtk_tooltips_set_tip (GTK_TOOLTIPS (priv->tooltips),
-			      priv->throbber, str, str);
+			      ebox, str, str);
 
-	g_signal_connect (priv->throbber,
+	g_signal_connect (ebox,
 			  "button-press-event",
 			  G_CALLBACK (app_throbber_button_press_event_cb),
 			  NULL);
@@ -695,7 +699,7 @@ app_setup (GossipSession        *session,
 		gtk_window_set_default_size (GTK_WINDOW (priv->window), w, h);
 	}
 
-	if (x >= 1 && y >= 1) {
+	if (x >= 0 && y >= 0) {
 		/* Let the window manager position it if we
 		 * don't have good x, y coordinates.
 		 */
@@ -1220,7 +1224,7 @@ app_room_join_favorites_cb (GtkWidget *window,
 
 	priv = GET_PRIV (app);
 
-	gossip_chatroom_manager_join_favourites (priv->chatroom_manager);
+	gossip_session_chatroom_join_favorites (priv->session);
 }
 
 static void
@@ -1370,7 +1374,7 @@ app_help_contents_cb (GtkWidget *window,
 }
 
 static gboolean
-app_throbber_button_press_event_cb (GtkWidget      *throbber,
+app_throbber_button_press_event_cb (GtkWidget      *throbber_ebox,
 				    GdkEventButton *event,
 				    gpointer        user_data)
 {
@@ -1391,14 +1395,14 @@ app_session_protocol_connecting_cb (GossipSession  *session,
 				    gpointer        user_data)
 {
 	GossipAppPriv *priv;
-	const gchar   *id;
+	const gchar   *name;
 
 	priv = GET_PRIV (app);
 
-	id = gossip_account_get_id (account);
-	gossip_debug (DEBUG_DOMAIN_SESSION, "Connecting account:'%s'", id);
+	name = gossip_account_get_name (account);
+	gossip_debug (DEBUG_DOMAIN_SESSION, "Connecting account:'%s'", name);
 
-	gossip_throbber_start (GOSSIP_THROBBER (priv->throbber));
+	ephy_spinner_start (EPHY_SPINNER (priv->throbber));
 }
 
 static void
@@ -1409,12 +1413,12 @@ app_session_protocol_connected_cb (GossipSession  *session,
 {
 	GossipAppPriv *priv;
 	gboolean       connecting;
-	const gchar   *id;
+	const gchar   *name;
 
 	priv = GET_PRIV (app);
 
-	id = gossip_account_get_id (account);
-	gossip_debug (DEBUG_DOMAIN_SESSION, "Connected account:'%s'", id);
+	name = gossip_account_get_name (account);
+	gossip_debug (DEBUG_DOMAIN_SESSION, "Connected account:'%s'", name);
 
 	gossip_session_count_accounts (priv->session,
 				       NULL,
@@ -1422,7 +1426,7 @@ app_session_protocol_connected_cb (GossipSession  *session,
 				       NULL);
 
 	if (connecting < 1) {
-		gossip_throbber_stop (GOSSIP_THROBBER (priv->throbber));
+		ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
 	}
 
 	g_hash_table_remove (priv->errors, account);
@@ -1470,15 +1474,15 @@ app_session_protocol_disconnected_cb (GossipSession  *session,
 	GossipAppPriv *priv;
 	gboolean       connecting;
 	gboolean       should_reconnect;
-	const gchar   *id;
+	const gchar   *name;
 #ifdef HAVE_DBUS
 	gboolean       nm_connected;
 #endif
 
 	priv = GET_PRIV (app);
 
-	id = gossip_account_get_id (account);
-	gossip_debug (DEBUG_DOMAIN_SESSION, "Disconnected account:'%s'", id);
+	name = gossip_account_get_name (account);
+	gossip_debug (DEBUG_DOMAIN_SESSION, "Disconnected account:'%s'", name);
 
 	gossip_session_count_accounts (priv->session,
 				       NULL,
@@ -1486,18 +1490,19 @@ app_session_protocol_disconnected_cb (GossipSession  *session,
 				       NULL);
 
 	if (connecting < 1) {
-		gossip_throbber_stop (GOSSIP_THROBBER (priv->throbber));
+		ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
 	}
 
 	app_connection_items_update ();
 	app_favorite_chatroom_menu_update ();
 	app_presence_updated ();
 
-	should_reconnect = reason != GOSSIP_DISCONNECT_ASKED;
+	should_reconnect = reason != GOSSIP_PROTOCOL_DISCONNECT_ASKED;
 
 #ifdef HAVE_DBUS
 	/* If NM says we are offline that's useless to retry to connect,
-	 * NM will tell us when network is up again. */
+	 * NM will tell us when network is up again. 
+	 */
 	if (gossip_dbus_nm_get_state (&nm_connected)) {
 		should_reconnect &= nm_connected;
 	}
@@ -1527,12 +1532,12 @@ app_session_protocol_error_cb (GossipSession  *session,
 {
 	GossipAppPriv *priv;
 	gboolean       connecting;
-	const gchar   *id;
+	const gchar   *name;
 
 	priv = GET_PRIV (app);
 
-	id = gossip_account_get_id (account);
-	gossip_debug (DEBUG_DOMAIN_SESSION, "Error for account:'%s'", id);
+	name = gossip_account_get_name (account);
+	gossip_debug (DEBUG_DOMAIN_SESSION, "Error for account:'%s'", name);
 
 	gossip_session_count_accounts (priv->session,
 				       NULL,
@@ -1540,7 +1545,7 @@ app_session_protocol_error_cb (GossipSession  *session,
 				       NULL);
 
 	if (connecting < 1) {
-		gossip_throbber_stop (GOSSIP_THROBBER (priv->throbber));
+		ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
 	}
 
 	app_accounts_error_display (account, error);
@@ -1645,7 +1650,7 @@ gossip_app_toggle_visibility (void)
 			gtk_window_set_default_size (GTK_WINDOW (priv->window), w, h);
 		}
 
-		if (x >= 1 && y >= 1) {
+		if (x >= 0 && y >= 0) {
 			/* Let the window manager position it if we
 			 * don't have good x, y coordinates.
 			 */
@@ -2039,7 +2044,7 @@ app_idle_check_cb (GossipApp *app)
 		 state == GOSSIP_PRESENCE_STATE_EXT_AWAY) {
 		/* Allow some slack before returning from away. */
 		if (idle >= -BACK_SLACK && idle <= 0) {
-			gossip_debug (DEBUG_DOMAIN_IDLE, "Slack, do nothing.");
+			/* gossip_debug (DEBUG_DOMAIN_IDLE, "Slack, do nothing."); */
 			app_status_flash_start ();
 		}
 		else if (idle < -BACK_SLACK) {
@@ -2048,7 +2053,7 @@ app_idle_check_cb (GossipApp *app)
 			return TRUE;
 		}
 		else if (idle > BACK_SLACK) {
-			gossip_debug (DEBUG_DOMAIN_IDLE, "Don't interrupt break.");
+			/* gossip_debug (DEBUG_DOMAIN_IDLE, "Don't interrupt break."); */
 			app_status_flash_stop ();
 		}
 	}
@@ -2157,16 +2162,14 @@ gossip_app_net_up (void)
 }
 
 void
-gossip_app_create (GossipSession        *session,
-		   GossipAccountManager *manager)
+gossip_app_create (GossipSession *session)
 
 {
 	g_return_if_fail (GOSSIP_IS_SESSION (session));
-	g_return_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (manager));
 
 	g_object_new (GOSSIP_TYPE_APP, NULL);
 
-	app_setup (session, manager);
+	app_setup (session);
 }
 
 GossipApp *
@@ -2773,10 +2776,10 @@ app_status_icon_flash_maybe_stop (void)
 }
 
 static void
-app_chatroom_auto_connect_cb (GossipChatroomManager    *manager,
-			      GossipChatroomProvider   *provider,
-			      GossipChatroom           *chatroom,
-			      gpointer                  user_data)
+app_session_chatroom_auto_connect_cb (GossipSession            *session,
+				      GossipChatroomProvider   *provider,
+				      GossipChatroom           *chatroom,
+				      gpointer                  user_data)
 {
 	GossipGroupChat *chat;
 
@@ -2785,7 +2788,6 @@ app_chatroom_auto_connect_cb (GossipChatroomManager    *manager,
 		      gossip_chatroom_get_name (chatroom));
 
 	chat = gossip_group_chat_new (provider, chatroom);
-/* 	g_object_unref (chat); */
 }
 
 static void
