@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004 Imendio AB
+ * Copyright (C) 2004-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,6 +16,8 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
 #ifndef __GOSSIP_CHAT_MANAGER_H__
@@ -25,6 +27,8 @@
 
 #include <libgossip/gossip-contact.h>
 
+G_BEGIN_DECLS
+
 #define GOSSIP_TYPE_CHAT_MANAGER         (gossip_chat_manager_get_type ())
 #define GOSSIP_CHAT_MANAGER(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GOSSIP_TYPE_CHAT_MANAGER, GossipChatManager))
 #define GOSSIP_CHAT_MANAGER_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k), GOSSIP_TYPE_CHAT_MANAGER, GossipChatManagerClass))
@@ -45,15 +49,15 @@ struct _GossipChatManagerClass {
 	GObjectClass parent_class;
 };
 
-
 GType              gossip_chat_manager_get_type       (void) G_GNUC_CONST;
+GossipChatManager *gossip_chat_manager_new       (void);
+GossipPrivateChat *gossip_chat_manager_get_chat  (GossipChatManager *manager,
+						  GossipContact     *contact);
+GList *            gossip_chat_manager_get_chats (GossipChatManager *manager);
+void               gossip_chat_manager_show_chat (GossipChatManager *manager,
+						  GossipContact     *contact);
 
-GossipChatManager *gossip_chat_manager_new            (void);
-GossipPrivateChat *gossip_chat_manager_get_chat       (GossipChatManager *manager,
-						       GossipContact     *contact);
-GList *            gossip_chat_manager_get_chats      (GossipChatManager *manager);
-void               gossip_chat_manager_show_chat      (GossipChatManager *manager,
-						       GossipContact     *contact);
+G_END_DECLS
 
 #endif /* __GOSSIP_CHAT_MANAGER_H__ */
 
