@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2005 Imendio AB
+ * Copyright (C) 2003-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,6 +16,10 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
  */
 
 #ifndef __GOSSIP_CONTACT_INFO_DIALOG_H__
@@ -23,7 +27,11 @@
 
 #include <libgossip/gossip-contact.h>
 
+G_BEGIN_DECLS
+
 void gossip_contact_info_dialog_show (GossipContact *contact,
 				      GtkWindow     *parent);
 
+G_END_DECLS
+
 #endif /* __GOSSIP_CONTACT_INFO_DIALOG_H__ */
