@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2004 Imendio AB
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,15 +16,21 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
  */
 
 #ifndef __GOSSIP_GROUP_CHAT_H__
 #define __GOSSIP_GROUP_CHAT_H__
 
+#include <libgossip/gossip-chatroom.h>
 #include <libgossip/gossip-chatroom-provider.h>
 
 #include "gossip-chat.h"
-#include "gossip-ui-utils.h"
+
+G_BEGIN_DECLS
 
 #define GOSSIP_TYPE_GROUP_CHAT         (gossip_group_chat_get_type ())
 #define GOSSIP_GROUP_CHAT(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GOSSIP_TYPE_GROUP_CHAT, GossipGroupChat))
@@ -47,11 +53,15 @@ struct _GossipGroupChatClass {
 	GossipChatClass parent_class;
 };
 
-GType            gossip_group_chat_get_type    (void) G_GNUC_CONST;
-
-GossipGroupChat *gossip_group_chat_show        (GossipChatroomProvider *provider,
-						GossipChatroomId        id);
-GossipChatroomId gossip_group_chat_get_room_id (GossipGroupChat        *groupchat);
+GType            gossip_group_chat_get_type              (void) G_GNUC_CONST;
+GossipGroupChat *gossip_group_chat_new                   (GossipChatroomProvider *provider,
+							  GossipChatroom         *chatroom);
+GossipChatroomId gossip_group_chat_get_chatroom_id       (GossipGroupChat        *group_chat);
+GossipChatroomProvider *
+		 gossip_group_chat_get_chatroom_provider (GossipGroupChat        *group_chat);
+GossipChatroom * gossip_group_chat_get_chatroom          (GossipGroupChat        *group_chat);
+void             gossip_group_chat_set_topic             (GossipGroupChat        *group_chat);
 
+G_END_DECLS
 
 #endif /* __GOSSIP_GROUP_CHAT_H__ */
