@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Kevin Dougherty <gossip@kdough.net>
+ * Copyright (C) 2003-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,14 +16,18 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Kevin Dougherty <gossip@kdough.net>
  */
 
 #ifndef __GOSSIP_IDLE_H__
 #define __GOSSIP_IDLE_H__
 
-#include <glib.h>
+G_BEGIN_DECLS
 
-gint32		gossip_idle_get_seconds	    (void);
-void            gossip_idle_reset           (void);
+gint32 gossip_idle_get_seconds (void);
+void   gossip_idle_reset       (void);
+
+G_END_DECLS
 
 #endif /*  __GOSSIP_IDLE_H__ */
