@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2006 Imendio AB
- * Copyright (C) 2002-2003 Richard Hult <richard@imendio.com>
- * Copyright (C) 2005      Martyn Russell <mr@gnome.org>
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,26 +16,30 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
+ * 
+ *          Part of this file is copied from GtkSourceView (gtksourceiter.c):
+ *          Paolo Maggi
+ *          Jeroen Zwartepoorte
  */
 
 #ifndef __GOSSIP_UI_UTILS_H__
 #define __GOSSIP_UI_UTILS_H__
 
-#include <gtk/gtkoptionmenu.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtktextview.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtksizegroup.h>
-#include <glade/glade.h>
-
 #include <libgossip/gossip-account.h>
 #include <libgossip/gossip-contact.h>
+#include <libgossip/gossip-vcard.h>
 #include <libgossip/gossip-presence.h>
 #include <libgossip/gossip-chatroom.h>
-#include <libgossip/gossip-vcard.h>
+#include <libgossip/gossip-avatar.h>
 
 #include "gossip-chat-view.h"
 
+G_BEGIN_DECLS
+
 /* Glade */
 void       gossip_glade_get_file_simple             (const gchar         *filename,
 						     const gchar         *root,
@@ -94,8 +96,7 @@ GdkPixbuf *gossip_pixbuf_for_contact                (GossipContact       *contac
 GdkPixbuf *gossip_pixbuf_offline                    (void);
 GdkPixbuf *gossip_pixbuf_for_chatroom_status        (GossipChatroom      *chatroom,
 						     GtkIconSize          icon_size);
-GdkPixbuf *gossip_pixbuf_from_avatar_scaled         (const guchar        *avatar,
-						     gsize                len,
+GdkPixbuf *gossip_pixbuf_from_avatar_scaled         (GossipAvatar        *avatar,
 						     gint                 width,
 						     gint                 height);
 GdkPixbuf *gossip_pixbuf_avatar_from_vcard          (GossipVCard         *vcard);
@@ -127,9 +128,15 @@ void       gossip_window_set_default_icon_name      (const gchar         *name);
 void       gossip_url_show                          (const char          *url);
 void       gossip_help_show                         (void);
 
+void       gossip_toggle_button_set_state_quietly   (GtkWidget           *widget,
+						     GCallback            callback,
+						     gpointer             user_data,
+						     gboolean             active);
 GtkWidget *gossip_link_button_new                   (const gchar         *url,
 						     const gchar         *title);
 
 void       gossip_request_user_attention            (void);
 
+G_END_DECLS
+
 #endif /*  __GOSSIP_UI_UTILS_H__ */
